﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.BusinessLogicContracts;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.BusinessLogic
{
    public class EmployeeReportLogic : IEmployeeReportLogic
    {       
        private readonly IPCStorage _pcStorage;
        private readonly IProductStorage _productStorage;
        private readonly IConsignmentStorage _consignmentStorage;
        private readonly IOrderStorage _orderStorage;
        
        public EmployeeReportLogic(IPCStorage pcStorage, IProductStorage productStorage, IConsignmentStorage consignmentStorage, IOrderStorage orderStorage)
        {
            _pcStorage = pcStorage;
            _productStorage = productStorage;
            _consignmentStorage = consignmentStorage;
            _orderStorage = orderStorage;
        }

        public List<ReportPCByDateViewModel> GetPCsByDate(ReportDateBindingModel model)
        {
            return _pcStorage.GetFilteredList(new PCSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            })
            .Select(x => new ReportPCByDateViewModel
            {
                PCName = x.Name,
                Components = x.PCComponents.Values.Select(x => (x.Item1.Name, x.Item2)).ToList(),
                EmployeeUsername = x.EmployeeUsername
            }).ToList();

        }

        public List<ReportProductByDateViewModel> GetProductsByDate(ReportDateBindingModel model)
        {
            return _productStorage.GetFilteredList(new ProductSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            })
            .Select(x => new ReportProductByDateViewModel
            {
                ProductName = x.Name,
                Components = x.ProductComponents.Values.Select(x => (x.Item1.Name, x.Item2)).ToList()

            }).ToList();
        }

        public List<ReportConsignmentsViewModel> GetConsignmentsByComponents(ReportComponentsBindingModel model)
        {
            var list = new List<ReportConsignmentsViewModel>();

            Dictionary<int, IProductModel> specifiedProducts = new Dictionary<int, IProductModel>();
            
            foreach(var component in model.Components)
            {
                var products = _productStorage.GetFilteredList(new ProductSearchModel
                {
                    ComponentID = component.ID
                });

                foreach(var product in products)
                {
                    if(!specifiedProducts.ContainsKey(product.ID))
                    {
                        specifiedProducts.Add(product.ID, product);
                    }
                }
            }

            Dictionary<int, IConsignmentModel> specifiedConsignments = new Dictionary<int, IConsignmentModel>();

            foreach (var product in specifiedProducts)
            {
                var consignments = _consignmentStorage.GetFilteredList(new ConsignmentSearchModel
                {
                    ProductID = product.Key
                });

                if(!specifiedConsignments.Any())
                {
                    foreach (var consignment in consignments)
                    {
                        specifiedConsignments.Add(consignment.ID, consignment);
                    }
                }

                foreach(var consignment in specifiedConsignments.Values)
                {
                    if(!consignments.Contains(consignment))
                    {
                        specifiedConsignments.Remove(consignment.ID);
                    }
                }
                
            }

            foreach(var consignment in specifiedConsignments)
            {
                var record = new ReportConsignmentsViewModel
                {
                    ID = consignment.Key,
                    Products = consignment.Value.ConsignmentProducts.Values.Select(x => (x.Item1.Name,x.Item2)).ToList()
                };
                list.Add(record);
            }

            return list;
        }

        public void SaveProductsToPdfFile(ReportDateBindingModel model)
        {
            //will be implemented in the future!
        }

        public void SavePCsToPdfFile(ReportDateBindingModel model)
        {
            //will be implemented in the future!
        }

        
    }
}
