﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreDatabaseImplement.Implements
{
    public class RequestStorage : IRequestStorage
    {
        public RequestViewModel? GetElement(RequestSearchModel model)
        {
            using var context = new ComputerStoreDatabase();
            return context.Requests
                .Include(x => x.Order)
                .Include(y => y.PCs)
                .FirstOrDefault(x => 
                (!model.ID.HasValue || x.ID == model.ID) && 
                (!model.OrderID.HasValue || x.OrderID == model.OrderID) && 
                (!model.PCID.HasValue || x.PCID == model.PCID))?.GetViewModel;
        }

        public List<RequestViewModel> GetFilteredList(RequestSearchModel model)
        {
            using var context = new ComputerStoreDatabase();
            return context.Requests
                .Include(x => x.Order)
                .Include(y => y.PCs)
                .Where(x =>
                (!model.ID.HasValue || x.ID == model.ID) &&
                (!model.OrderID.HasValue || x.OrderID == model.OrderID) &&
                (!model.PCID.HasValue || x.PCID == model.PCID))
                .Select(y => y.GetViewModel)
                .ToList();
        }

        public List<RequestViewModel> GetFullList()
        {
            using var context = new ComputerStoreDatabase();
            return context.Requests
                .Include(x => x.Order)
                .Include(y => y.PCs)
                .Select(y => y.GetViewModel)
                .ToList();
        }

        public RequestViewModel? Insert(RequestBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var newRequest = Request.Create(context, model);
            if (newRequest == null) { return null; }
            context.Requests.Add(newRequest);
            context.SaveChanges();
            return context.Requests
                .Include(x => x.Order)
                .Include(y => y.PCs)
                .FirstOrDefault(x => x.ID == model.ID)?.GetViewModel;
        }

        public RequestViewModel? Update(RequestBindingModel model)
        {
            using var context = new ComputerStoreDatabase();
            var request = context.Requests.FirstOrDefault(x => x.ID == model.ID);
            if (request == null)
            {
                return null;
            }

            request.Update(model);
            context.SaveChanges();

            return context.Requests
                .Include(x => x.Order)
                .Include(y => y.PCs)
                .FirstOrDefault(x => x.ID == model.ID)?.GetViewModel;
        }

        public RequestViewModel? Delete(RequestBindingModel model)
        {
            using var context = new ComputerStoreDatabase();

            var request = context.Requests
                .Include(x => x.Order)
                .Include(y => y.PCs)
                .FirstOrDefault(x => x.ID == model.ID);

            if (request != null)
            {
                context.Requests.Remove(request);
                context.SaveChanges();

                return request.GetViewModel;
            }

            return null;
        }
    }
}
